""" Base (abstract) class for all AST nodes
    @author: Marc Provost
    @organization: McGill University
    @license: GNU General Public License
    @contact: marc.provost@mail.mcgill.ca    
"""

from StrReprVisitor import StrReprVisitor
from utility.TypeCheckUtility import TypeCheckUtility
typeChecker=TypeCheckUtility.getInstance()

class ASTNode:
  """ Base (abstract) class for all AST nodes such as Number,Functions,etc.
  """
  def __init__(self):
    """ This class is abstract, and error is raised if an attempt is made
        to instantiate it.
    """
    if self.__class__ == ASTNode:
        raise NotImplementedError,"abstract class cannot be instantiated"

  def accept(self,visitor):
    """ Method used in the visitor pattern.
        Calls the appropriate action to perform while
        visiting this object.
        @param visitor: Visitor accepted by this object
        @type visitor: Visitor
    """
    pass


  def __str__(self):
    """ String representation of the complete AST (common to every child)
        @rtype: string
    """  
    strVisitor=StrReprVisitor()
    self.accept(strVisitor)
    return strVisitor.getStrRepr()
       
  def clone(self):
    """ Returns a copy of this object
        @rtype: ASTNode
    """     
    pass
  
  def equals(self, anObject):
    """ Returns true if this object equals another object
        @param anObject: Object compared to this object
        @type anObject: Object
    """  
    pass