""" Number node of the AST.
    @author: Marc Provost
    @organization: McGill University
    @license: GNU General Public License
    @contact: marc.provost@mail.mcgill.ca    
"""

from ASTVisitor import ASTVisitor
from ASTNode import ASTNode
from utility.TypeCheckUtility import TypeCheckUtility
typeChecker=TypeCheckUtility.getInstance()

class Number(ASTNode):
  """ Encapsulate numerical data (float).
      Instance of this class are leaf nodes of the AST
      @group Accessors: set*, get*
      @sort: __init__, set*, get*
  """
 

  def __init__(self, number, numParen=0):
    """ Number constructor
        @param number: numerical value encapsulated by this object
        @param numParen: number of parenthesis
        @type number: int, float
        @type numParen: int        
    """
    if typeChecker.debug: typeChecker.typeCheck([number], [(int,float)])
     # explicit casting into "float"
    self.__value = float(number) 
    #Number of parenthesis around
    self.__numParen=numParen    


  def getValue(self):
    """ Returns the encapsulated value
        @rtype: float
    """      
    return self.__value

  def getNumParen(self):
    """ Returns the number of parenthesis around this ASTNode
        @rtype: int
    """
    return self.__numParen        
    
  def setValue(self,number):
    """ Set the value of this Number
        @param number: The value to be encapsulated
        @type number: int, float
    """      
    if typeChecker.debug: typeChecker.typeCheck([number], [(int,float)])
    self.__value=float(number)
    
  def setNumParen(self, numParen):
    """ Set the number of parenthesis around this number
        @param numParen: number of parenthesis
        @type numParen: int
    """
    self.__numParen=numParen    


  def accept(self,visitor):
    if typeChecker.debug: typeChecker.typeCheck([visitor], [ASTVisitor])
    visitor.visitNumber(self)
  
  def clone(self):
    """ Returns a copy of this object
        @rtype: L{Number}
    """      
    return Number(self.__value,self.__numParen)

  def equals(self, anObject):
    return isinstance(anObject,Number) and\
           anObject.getValue()==self.__value and\
           anObject.getNumParen()==self.__numParen