from formulaparser.ParseNode import *

%%
parser FormulaParser:
    ignore:    "[ \r\t\n]+"

    token int: "[0-9]+"
    token real: "([0-9]+[.][0-9]*)|([0-9]*[.][0-9]+)"
    token number: "([0-9]+)|(([0-9]+[.][0-9]*)|([0-9]*[.][0-9]+))"
    token char: "[a-zA-Z]"
    token string: "[0-9]*[a-zA-Z]+([a-zA-Z]|[0-9])*"
    token ref: "[$]?[A-Z#][A-Z]?[A-Z]?[$]?[1-9#][0-9]?[0-9]?[0-9]?"
    token rangeref: "[$]?[A-Z#][A-Z]?[A-Z]?[$]?[1-9#][0-9]?[0-9]?[0-9]?[:][$]?[A-Z#][A-Z]?[A-Z]?[$]?[1-9#][0-9]?[0-9]?[0-9]?"
    token add: "\+"
    token sub: "-"
    token mul: "\*"
    token div: "/"
    token expop: "\^"
    token funcname: "[a-zA-Z]+"
    token lpar: "\("
    token rpar: "\)"
    token comma: ","
    token equal: "="
    token empty: ""


    token end: ";"
    token NUM: "[0-9]+"
    token VAR: "[a-zA-Z_]+"


    rule GOAL:            FORMULA end                     {{return FORMULA}}


    rule EXP:               TERM                          {{parseNode=ParseNode("exp")}}
                                                          {{parseNode.children.append(TERM)}}

                                ((add TERM)               {{parseNode.children.append(ParseNode("add"))}}
                                                          {{parseNode.children.append(TERM)}}
                                                                           
                                  |  (sub TERM)           {{parseNode.children.append(ParseNode("sub"))}}
                                                          {{parseNode.children.append(TERM)}}
                                                                          
                                )*                        {{return parseNode}}

    rule TERM:          EXPTERM                           {{parseNode=ParseNode("term")}}
                                                          {{parseNode.children.append(EXPTERM)}}
                                                                         
                               ((mul EXPTERM)             {{parseNode.children.append(ParseNode("mul"))}}
                                                          {{parseNode.children.append(EXPTERM)}}
                                                                         
                                 |  (div EXPTERM)         {{parseNode.children.append(ParseNode("div"))}}
                                                          {{parseNode.children.append(EXPTERM)}}
                                                                         
                               )*                         {{return parseNode}}

    rule EXPTERM:   SIGNEDFACTOR                          {{parseNode=ParseNode("expterm")}}
                                                          {{parseNode.children.append(SIGNEDFACTOR)}}
                                                                         
                               (expop SIGNEDFACTOR        {{parseNode.children.append(ParseNode("exponent"))}}
                                                          {{parseNode.children.append(SIGNEDFACTOR)}}
                                                                         
                               )*                         {{return parseNode}}

    rule SIGNEDFACTOR:   FACTOR                           {{return FACTOR}}
                                         |
                                          (sub FACTOR)    {{parseNode=ParseNode("signedfactor")}}
                                                          {{parseNode.children.append(FACTOR)}}
                                                          {{return parseNode}}

    rule FACTOR:        lpar EXP rpar                     {{parseNode=ParseNode("parenExp")}} 
                                                          {{parseNode.children.append(EXP)}}                                                         
                                                          {{return parseNode}}
                                | int                     {{return ParseNode("number",int)}}
                                | real                    {{return ParseNode("number",real)}}
                                | ref                     {{return ParseNode("ref", ref)}}
                                | FUNCTION                {{return FUNCTION}}


    rule ARG:                                             {{parseNode=ParseNode("arg")}}

                               (rangeref                  {{parseNode.children.append(ParseNode("rangeref", rangeref))}}
                              |  EXP)                     {{parseNode.children.append(EXP)}}
                                    (comma
                                        (rangeref         {{parseNode.children.append(ParseNode("rangeref", rangeref))}}
                                        | EXP)            {{parseNode.children.append(EXP)}}
                                    )*                    {{return parseNode}}

    rule FUNCTION:      funcname                          {{parseNode=ParseNode("function", funcname)}}
                                                          {{ARG=None}}
                                   lpar
                                   [ARG]                  {{if ARG!=None:parseNode.children.append(ARG)}}
                                   rpar
                                                          {{return parseNode}}

    rule FORMULA:       '='EXP                            {{return EXP}}

%%
if __name__=='__main__':
    print 'Welcome to the formula parser!'
    # We could have put this loop into the parser, by making the
    # `goal' rule use (expr | set var expr)*, but by putting the
    # loop into Python code, we can make it interactive (i.e., enter
    # one expression, get the result, enter another expression, etc.)
    while 1:
        s = raw_input('>>> ')
        print s
        parse('GOAL', s)